/*
** Module   : ParmType
** Abstract : Parameter types enumerator.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130304 Created initial version.
** 002 CA  20130313 Added TEXT_VARARGS.
** 003 SVL 20140106 Added fromClass function.
*/

package com.goldencode.p2j.convert;

import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.util.*;

/**
 * Encodes the possible parameter types in Progress. This includes a code for each
 * individual wrapper type.  But it also includes codes for the common parent types
 * for use in cases where the parameter can be one of multiple related types (e.g.
 * NUMBERTYPE</action> would indicate that an <code>INTEGER</code>,
 * <code>INT64</code> or <code>DECIMAL</code> could all be used as a parameter).
 */ 
public enum ParmType
{
   CHAR(character.class) 
      { @Override public String toString() { return "character"; } },
   LC(longchar.class)
      { @Override public String toString() { return "longchar"; } },
   CLOB(null)
      { @Override public String toString() { return "CLOB"; } },
   LARGE_CHAR_OBJECT(null)
      { @Override public String toString() { return "LARGE_CHAR_OBJECT"; } },
   TEXT(Text.class)
      { @Override public String toString() { return "Text"; } },
   TEXT_VARARGS(Text[].class)
      { @Override public String toString() { return "Text[]"; } },
   INT(integer.class)
      { @Override public String toString() { return "integer"; } },
   DEC(decimal.class)
      { @Override public String toString() { return "decimal"; } },
   INT64(int64.class)
      { @Override public String toString() { return "int64"; } },
   NUM(NumberType.class)
      { @Override public String toString() { return "NumberType"; } },
   NUM_VARARGS(NumberType[].class)
      { @Override public String toString() { return "NumberType[]"; } },
   LOG(logical.class)
      { @Override public String toString() { return "logical"; } },
   DATE(date.class)
      { @Override public String toString() { return "date"; } },
   DT(datetime.class)
      { @Override public String toString() { return "datetime"; } },
   DTTZ(datetimetz.class)
      { @Override public String toString() { return "datetimetz"; } },
   HANDLE(handle.class)
      { @Override public String toString() { return "handle"; } },
   WID(GenericWidget.class)
      { @Override public String toString() { return "GenericWidget"; } },
   RECID(recid.class)
      { @Override public String toString() { return "recid"; } },
   RECORD(DataModelObject.class)
      { @Override public String toString() { return "record"; } },
   ROWID(rowid.class)
      { @Override public String toString() { return "rowid"; } },
   ROWID_ARRAY(rowid[].class)
      { @Override public String toString() { return "rowid[]"; } },
   MEM(memptr.class)
      { @Override public String toString() { return "memptr"; } },
   RAW(raw.class)
      { @Override public String toString() { return "raw"; } },
   BIN(BinaryData.class)
      { @Override public String toString() { return "BinaryData"; } },
   BDT(BaseDataType.class)
      { @Override public String toString() { return "BaseDataType"; } },
   BDT_ARRAY(BaseDataType[].class)
      { @Override public String toString() { return "BaseDataType[]"; } },
   BDT_VARARGS(BaseDataType[].class)
      { @Override public String toString() { return "BaseDataType[]"; } };
   
   /** 
    * The P2J wrapper class for this type (may be an array with the component a P2J wrapper type.
    */
   private Class<?> cls;
   
   /**
    * Build a parameter type by specifying its wrapper class. May be an array class.
    * 
    * @param    cls
    *           The parameter's wrapper class (may be an array class).
    */
   private ParmType(Class<?> cls)
   {
      this.cls = cls;
   }
   
   /**
    * Check if the candidate and expected types match.
    * 
    * @param    candidate
    *           The candidate type. May be null.
    * @param    expected
    *           The expected type. May not be null.
    *           
    * @return   <code>true</code> if they match (i.e. the expected is a super-class of candidate)
    */
   public static boolean typesMatch(ParmType candidate, ParmType expected)
   {
      if (candidate == null)
      {
         return false;
      }

      Class<?> cls1 = candidate.cls;
      Class<?> cls2 = expected.cls;
      // if one of the classes is unknown or they are (or not) both array, then they do not match
      if (cls1 == null || cls2 == null || cls1.isArray() != cls2.isArray())
      {
         return false;
      }
      
      if (cls1.isArray() && cls2.isArray())
      {
         cls1 = cls1.getComponentType();
         cls2 = cls2.getComponentType();
      }
      
      return cls2.isAssignableFrom(cls1);
   }

   /**
    * Resolve the {@link ParmType constant} from the string representation of the given wrapper
    * type.
    * 
    * @param    ptype
    *           The name of the class of a P2J wrapper type.
    *           
    * @return   The enum constant for this wrapper type or null if not found.
    */
   public static ParmType fromString(String ptype)
   {
      for (ParmType pt : ParmType.values())
      {
         if (ptype.equals(pt.toString()))
            return pt;
      }
      
      return null;
   }

   /**
    * Resolve the {@link ParmType constant} from the class representing the given wrapper type.
    *
    * @param    ctype
    *           Class representing a P2J wrapper type.
    *
    * @return   The enum constant for this wrapper type or <code>null</code> if not found.
    */
   public static ParmType fromClass(Class ctype)
   {
      for (ParmType pt : ParmType.values())
      {
         if (ctype.equals(pt.cls))
            return pt;
      }

      return null;
   }

   /**
    * Check if the type is a varargs type.
    * 
    * @return   <code>true</code> if the type is a vararg.
    */
   public boolean isVarargs()
   {
      return this == BDT_VARARGS || this == NUM_VARARGS || this == TEXT_VARARGS;
   }
}